(function($) {

  /**
   * Apply all the right classes to open the given group.
   *
   * @param $group
   */
  var group_open = function($group) {
    var $html = $('html');
    if ($html.data('popup_field_group_active_group').length) {
      group_close($($html.data('popup_field_group_active_group')));
    }
    $group.addClass('popup-field-group-open');
    $group.removeClass('popup-field-group-closed');
    $html
      .addClass('popup-field-group-active')
      .data('popup_field_group_active_group', $group.get(0));
  }

  /**
   * Apply all the right classes to close the given group.
   *
   * @param $group
   */
  var group_close = function($group) {
    $group.removeClass('popup-field-group-open');
    $group.addClass('popup-field-group-closed');
    $('html')
      .removeClass('popup-field-group-active')
      .data('popup_field_group_active_group', []);
  }

  /**
   * Close any open popup if ESC key is pressed.
   */
  $(document).keyup(function(e) {
    // Escape key
    if (e.keyCode == 27) {
      var $html = $('html');
      if ($html.data('popup_field_group_active_group')) {
        var $group = $($html.data('popup_field_group_active_group'));
        if ($group.length) {
          group_close($group);
        }
      }
    }
  });

  /**
   * Popup field groups Drupal behaviours.
   *
   * @type {Object}
   */
  Drupal.behaviors.popupFieldGroup = {

    /**
     * Attach our behaviors to the DOM.
     *
     * @param context
     * @param settings
     */
    attach: function(context, settings) {
      $(context).find('.popup-field-group').not('.popup-field-group-attached').each(function() {
        var $group = $(this);
        // If we contain a form element with an error on it, do not attach.
        // This displays it in plain sight for total clarity.
        if ($group.find('.error').length) {
          return;
        }
        $group.addClass('popup-field-group-attached');
        var $overlay = $group.find('.overlay').first();
        // Create link
        var $link = $(Drupal.theme('popupFieldGroupLink', $group.data('popup-link-title'), $group.data('popup-link-classes')));
        $link.bind('click.popupFieldGroup', function() {
          group_open($group);
        }).insertBefore($group);
        $overlay.click(function() {
          group_close($group);
        });
        $group.find('.popup-field-group-close').click(function() {
          group_close($group);
        });
        group_close($group);
      });
    },

    /**
     * Detach our behaviors from the DOM.
     *
     * @param context
     * @param settings
     * @param trigger
     */
    detach: function(context, settings, trigger) {

      // We don't need to do anything when ajax handlers are searching for
      // valid field values.
      if (trigger === 'serialize') {
        return;
      }

      $(context).find('.popup-field-group-attached').each(function() {
        var $group = $(this);
        if ($group.hasClass('popup-field-group-open')) {
          group_close($group);
        }
        // In general there's no need to cause visual chaos by suddenly showing
        // all of our popups contents, so we leave it as it is. However, if
        // we're being moved we should cleanly and completely remove everything
        // because our attachBehaviors are going to be run on us in a bit.
        if (trigger === 'move') {
          $group.removeClass('popup-field-group-attached popup-field-group-open popup-field-group-closed');
        }
      });
      // In general there's no need to cause visual chaos by removing the link.
      $(context).find('.popup-field-group-link').unbind('click.popupFieldGroup');
      // If we're being moved we should cleanly and completely remove every-
      // thing because our attachBehaviors are going to be run on us in a bit.
      if (trigger === 'move') {
        $(context).find('.popup-field-group-link').remove();
      }
    }
  };

  /**
   * Markup for our little popup link we generate.
   *
   * @param label
   * @param classes
   * @return {String}
   */
  Drupal.theme.prototype.popupFieldGroupLink = function(label, classes) {
    classes = classes || '';
    return '<a class="popup-field-group-link ' + classes + '" href="javascript:void();" title="' + label + '">' + label + '</a>';
  }

})(jQuery);